/*
 * Copyright (C) 2002-2006 Michel Santos (michel@xspace.org)
 * Licensed under the Academic Free License version 3.0
 * Please see the LICENSE.txt for more information.
 *
 * Created on October 11, 2002, 6:10 PM
 */

package org.xspace.mx;

/** 
 * The Measurement interface connects the value of a measurement to a system of units.
 * It is the motivation behind the entire system.
 * 
 * @author Michel Santos
 * @since 0.1
 */
public interface Measurement {

  /**
   * Returns a new instance of a Measurement that is "equivalent" in value
   * to this Measurement.
   *
   * @return A new instance of a {@link Measurement} that is "equivalent" in value
   * to this Measurement.
   */  
  public Measurement copy();
  
  /** <p>The system of units being used by this Measurement.</p>
   *
   * <p>Knowledge of {@link SystemOfUnits} being used is vital for proper implementations of 
   * any of the arithmetic methods: {@link #plus(Measurement)}, {@link #minus(Measurement)}, {@link #mult(Measurement)}, {@link #div(Measurement)}.
   * Both Measurements must reference the same SystemOfUnits for a proper comparison.</p>
   *
   * <p>For example, if Person A weighs items in terms of kilograms, and Person B weighs
   * items in terms of pounds, then they must both agree on a common system of units
   * before they can properly compare weights.  Their decision could be to measure items
   * using the SI system of units, Imperial system of units, or perhaps some other one.
   *
   * @return Returns the {@link SystemOfUnits system of units} being used by this {@link Measurement}.
   * @see #getSystemDimensions()
   */  
  public SystemOfUnits getSystem();
  
  
  /** <p>An array of {@link org.xspace.mx.Unit}s within the {@link org.xspace.mx.SystemOfUnits} being used by this Measurement.</p>
   *
   * <p>This array of Dimensions is vital for proper implementations of 
   * any of the arithmetic methods: {@link #plus(Measurement)}, {@link #minus(Measurement)}, {@link #mult(Measurement)}, {@link #div(Measurement)}, 
   * {@link #pow(double)}, {@link #root(double)}.</p>
   *
   * <p>For example, if this Measurement measures the speed of five meters per second,
   * then this method would return an array of Dimensions that contains:
   * <ul>
   * <li>a Length Dimension with a power of 1</li>
   * <li>a Time Dimension with a power of -1</li>
   * </ul>
   * </p>
   *
   * @return Returns an array of {@link Unit}s being used by this {@link Measurement}.
   * @see #getSystem()
   */  
  public org.xspace.mx.Unit[] getSystemDimensions();

  /** <p>The internal value of this measurement.  Meant to be used 
   * in the implementations of the other methods.</p>
   * <p>For example, an implementation might store the value of the measurement
   * as a multiple of the "standard units" of the {@link SystemOfUnits} being used.</p>
   *
   * @return Returns the internal value of this {@link Measurement}.
   */  
  public double getSystemValue();
  
  /** The value of this measurement relative to the dimensions of the input measurement.
   *
   * @param multipleOf The Measurement to compared with
   * @return The value of this {@link Measurement} as a multiple of the input parameter.
   */  
  public double getValue(Measurement multipleOf); 
  
  /** <p>Adds the other Measurement to this Measurement.</p>
   *  <p><i>Depending on the implementation, this <b>may</b> or <b>may not</b> alter this object.</i></p>
   *
   * @param other The Measurement that is added to this Measurement
   * @return A {@link Measurement} object that reflects the result of this addition.
   * @throws MeasurementException if both Measurements cannot be added.  
   * <p>This could occur if they were not referring to the same {@link SystemOfUnits}.</p>
   * <p>This could occur if they did not possess the exact same array of {@link Unit}s.</p>
   * @see #getSystem()
   * @see #getSystemDimensions()
   */  
  public Measurement plus(Measurement other) throws MeasurementException;
  //public Measurement plus(double val, Measurement other) throws MeasurementException;

  
  /** <p>Subtracts the other Measurement to this Measurement.</p>
   *  <p><i>Depending on the implementation, this <b>may</b> or <b>may not</b> alter this object.</i></p>
   *
   * @param other The Measurement that is subtracted to this Measurement
   * @return A {@link Measurement} object that reflects the result of this subtraction.
   * @throws MeasurementException if both Measurements cannot be subtracted.
   * <p>This could occur if they were not referring to the same {@link SystemOfUnits}.</p>
   * <p>This could occur if they did not possess the exact same array of {@link Unit}s.</p>
   * @see #getSystem()
   * @see #getSystemDimensions()
   */  
  public Measurement minus(Measurement other) throws MeasurementException;  
  //public Measurement minus(double val, Measurement other) throws MeasurementException;

  
  /** <p>Multiplies the other Measurement to this Measurement.</p>
   *  <p><i>Depending on the implementation, this <b>may</b> or <b>may not</b> alter this object.</i></p>
   *
   * @param other The Measurement that is multiplied by this Measurement
   * @return A {@link Measurement} object that reflects the result of this multiplication
   */  
  public Measurement mult(Measurement other);
  //public Measurement mult(double val, Measurement other);
  
  /** <p>Multiplies this {@link Measurement} by a scalar value.  This does not affect the {@link Unit}s of the Measurement.</p>
   *
   *  <p><i>Depending on the implementation, this <b>may</b> or <b>may not</b> alter this object.</i></p>
   *
   * <p>For example, if the Measurement represents five meters per second (5 [m / s]),
   * and it is multiplied by 2, then the result will be ten meters per second (10 [m / s]).</p>
   *
   * @param scaling A scalar value that multiplies this Measurement
   * @return A {@link Measurement} object that reflects the result of this multiplication
   */  
  public Measurement mult(double scaling);
  
  /** <p>Divides this Measurement by the other Measurement.</p>
   *  <p><i>Depending on the implementation, this <b>may</b> or <b>may not</b> alter this object.</i></p>
   *
   * @param other The Measurement that divided into this Measurement
   * @return A {@link Measurement} object that reflects the result of this division
   */
  public Measurement div(Measurement other);
  //public Measurement div(double val, Measurement other);
  
  /** <p>Divides this {@link Measurement} by a scalar value.  This does not affect the {@link Unit}s of the Measurement.</p>
   *
   *  <p><i>Depending on the implementation, this <b>may</b> or <b>may not</b> alter this object.</i></p>
   *
   * <p>For example, if the Measurement represents five meters per second (5 [m / s]),
   * and it is divided by 2, then the result will be two-and-a-half meters per second (2.5 [m / s]).</p>
   *
   * @param scaling A scalar value that multiplies this Measurement
   * @return A {@link Measurement} object that reflects the result of this multiplication
   */
  public Measurement div(double scaling);
  
  /** <p>Raises this {@link Measurement} to the specified power.</p>
   *
   *  <p><i>Depending on the implementation, this <b>may</b> or <b>may not</b> alter this object.</i></p>
   *
   * <p>For example, if the Measurement represents five meters per second (5 [m / s]),
   * and it is raised to the power of two, the result will be twenty-five meters squared per second squared (25 [m^2 / s^2]).</p>
   *
   * @param power The power that this Measurement is raised to
   * @return A {@link Measurement} object that reflects the result of raising the {@link Measurement} to the desired power
   * @see #root(double)
   */  
  public Measurement pow(double power);
  /** <p>Tales the root this {@link Measurement} by the specified power.</p>
   *  <p><i>Depending on the implementation, this <b>may</b> or <b>may not</b> alter this object.</i></p>
   * <p>For example, if the Measurement represents nine square meters (9 [m^2]),
   * and the square root is taken (power = 2), the result will be three meters (3 [m]).</p>
   *
   * @param power The power that this Measurement is lowered by
   * @return A {@link Measurement} object that reflects the result of lowering the {@link Measurement} by the desired power
   * @see #pow(double)
   */  
  public Measurement root(double power);

  
  /** 
   * A description of the units of this Measurement
   *
   * @return A description of the units of this Measurement
   */  
  public String getDescription();

  /** 
   * An abbreviated description of the units of this Measurement
   *
   * @return An abbreviated description of the units of this Measurement
   */  
  public String getAbbr();
}
