/*
 * Copyright (C) 2002-2006 Michel Santos (michel@xspace.org)
 * Licensed under the Academic Free License version 3.0
 * Please see the LICENSE.txt for more information.
 *
 * Created on May 1, 2003, 1:24 PM
 *
 * 2004-01-16 v.0.5.3 Updated to be Serializable
 * 2004-04-23 v.0.5.4 
 * - Added isSameMeasurementType and checkCompatibility functions
 * - Added readObject and writeObject which makes past serializable objects incompatible
 */

package org.xspace.mx.physics.immutable;

import org.xspace.mx.SystemOfUnits;
import org.xspace.mx.Measurement;
import org.xspace.mx.MeasurementException;
import org.xspace.mx.Unit;
import org.xspace.mx.physics.SI;

/**
 * The class from which all other units in this package are derived.
 * This class provides most of the functionality/implementation for this package.
 *
 * <p>Internally, the measurements are stored as multiples of base units of the SI system (e.g. meters, seconds, kilograms, etc.)</p>
 *
 * @author Michel Santos
 * @since 0.5
 */

public class BaseUnit implements Measurement, java.io.Serializable {  
  static final long serialVersionUID = -6116857044784240587L;
  
  /**
   * Creates a unit-less Measurement equal to the input value
   */
  public BaseUnit(double value) {
    this.systemValue = value;
  }
  
  /**
   * Creates a unit-less Measurement equal to one (1)
   */
  public BaseUnit() {
    this(1);
  }

  public Measurement plus(Measurement other) throws org.xspace.mx.MeasurementException {
    BaseUnit copy = this.copyBaseUnit();
    return copy.plusInternal(other);
  }
  
  public Measurement minus(Measurement other) throws org.xspace.mx.MeasurementException {
    BaseUnit copy = this.copyBaseUnit();
    return copy.minusInternal(other);
  }
  
  public Measurement mult(double scaling) {
    BaseUnit copy = this.copyBaseUnit();
    return copy.multInternal(scaling);
  }
  
  public Measurement mult(Measurement other) {
    BaseUnit copy = this.copyBaseUnit();
    return copy.multInternal(other);    
  }
  
  public Measurement div(double scaling) {
    BaseUnit copy = this.copyBaseUnit();
    return copy.divInternal(scaling);
  }
  
  public Measurement div(Measurement other) {
    BaseUnit copy = this.copyBaseUnit();
    return copy.divInternal(other);
  }
  
  public Measurement pow(double power) {
    BaseUnit copy = this.copyBaseUnit();
    return copy.powInternal(power);
  }
  
  public Measurement root(double power) {
    BaseUnit copy = this.copyBaseUnit();
    return copy.rootInternal(power);
  }
  
  public Measurement copy() {
    return copyBaseUnit();
  }
  protected BaseUnit copyBaseUnit() {
    BaseUnit cc = new org.xspace.mx.physics.immutable.BaseUnit(this.getSystemValue());
    
    Unit[] oldDims = this.getSystemDimensions();
    Unit[] newDims = cc.getSystemDimensions();
    for (int i = 0; i < oldDims.length; i++) {
      newDims[i].setPower(oldDims[i].getPower());
    }
    
    return cc;
  }
  
  protected Measurement plusInternal(Measurement other) throws MeasurementException {
    checkCompatibility(other);
    
    // Having checked everything, add the values
    this.systemValue += other.getSystemValue();
    
    return this;
  }
  
  protected Measurement minusInternal(Measurement other) throws MeasurementException {
    checkCompatibility(other);
    
    // Having checked everything, subtract the values
    this.systemValue -= other.getSystemValue();
    
    return this;
  }
  
  
  protected Measurement multInternal(double scaling) {
    systemValue *= scaling;
    return this;
  }
  
  protected Measurement multInternal(Measurement other) {
    // Check the System being used
    if (this.getSystem() != other.getSystem()) {
      throw new MeasurementException("Both measurements are not of the same system of measurements!");
    }
    
    Unit[] otherDims = other.getSystemDimensions();
    
    // Having checked everything, multiply
    this.systemValue *= other.getSystemValue();
    for (int i = 0; i < dims.length; i++) {
      dims[i].setPower(dims[i].getPower() + otherDims[i].getPower());
    }
    
    return this;
  }
  
  protected Measurement divInternal(double scaling) {
    systemValue /= scaling;
    return this;
  }
  
  protected Measurement divInternal(Measurement other) {
    // Check the System being used
    if (this.getSystem() != other.getSystem()) {
      throw new MeasurementException("Both measurements are not of the same system of measurements!");
    }
    
    Unit[] otherDims = other.getSystemDimensions();
    
    // Having checked everything, divide
    this.systemValue /= other.getSystemValue();
    for (int i = 0; i < dims.length; i++) {
      dims[i].setPower(dims[i].getPower() - otherDims[i].getPower());
    }
    
    return this;
  }
  
  
  protected Measurement powInternal(double power) {
    this.systemValue = Math.pow(this.systemValue,  power);
    
    for (int i = 0; i < dims.length; i++) {
      dims[i].setPower(dims[i].getPower() * power);
    }
    
    return this;
  }
  
  protected Measurement rootInternal(double power) {
    this.systemValue = Math.pow(this.systemValue,  1.0/power);
    
    for (int i = 0; i < dims.length; i++) {
      dims[i].setPower(dims[i].getPower() / power);
    }
    
    return this;
  }  
  
  
  public String getAbbr() {return "";}  
  public String getDescription() {return "";}
  
  public SystemOfUnits getSystem() {
    return SI.getSystem();
  }

  /**
   * The internally stored value of this Measurement
   */
  protected transient double systemValue = 0;
  
  // protected Dimension[] dims = new Dimension[0];
  protected static final int DIM_LENGTH = 0;
  protected static final int DIM_TIME = 1;
  protected static final int DIM_MASS = 2;
  protected static final int DIM_ANGLE = 3;
  protected transient Unit[] dims = {new SI.Length(), new SI.Time(), new SI.Mass(), new SI.Angle()};
  
  public org.xspace.mx.Unit[] getSystemDimensions() {
    return dims;
  }
  
  public double getSystemValue() {
    return this.systemValue;
  }
  
  /**
   * Determine whether this measurement and the other measurement are comparable
   */
  public boolean isSameMeasurementType(Measurement other) {
    // Check the System being used
    if (this.getSystem() != other.getSystem()) {
      // throw new MeasurementException("Both measurements are not of the same system of measurements!");
      return false;
    }
    
    // Check the number of dimensions
    Unit[] otherDims = other.getSystemDimensions();
    if (dims.length != otherDims.length) {
      // throw new MeasurementException("Both measurements are not of the same dimensions!");
      return false;
    }
    
    // Check powers of each dimension    
    for (int i = 0; i < dims.length; i++) {
      // if (dims[i].getType() != otherDims[i].getType() || dims[i].getPower() != otherDims[i].getPower()) {
      if (dims[i].getType() != otherDims[i].getType() || Math.abs(dims[i].getPower() - otherDims[i].getPower()) > TOL) {
        // throw new MeasurementException("Both measurements do not have the same dimenion powers!");
        return false;
      }
    }
    
    return true;
  }
  
  /**
   * <p>Check the compatibility of this measurement and the other measurement</p>
   * 
   * <p>This is a sister function of isSameMeasurementType</p>
   */
  private void checkCompatibility(Measurement other) throws MeasurementException {
    // Check the System being used
    if (this.getSystem() != other.getSystem()) {
      throw new MeasurementException("Both measurements are not of the same system of measurements!");      
    }
    
    // Check the number of dimensions
    Unit[] otherDims = other.getSystemDimensions();
    if (dims.length != otherDims.length) {
      throw new MeasurementException("Both measurements are not of the same dimensions!");      
    }
    
    // Check powers of each dimension    
    /** @todo Consider a more rigorous check */
    for (int i = 0; i < dims.length; i++) {
      // if (dims[i].getType() != otherDims[i].getType() || dims[i].getPower() != otherDims[i].getPower()) {
      if (dims[i].getType() != otherDims[i].getType() || Math.abs(dims[i].getPower() - otherDims[i].getPower()) > TOL) {
        throw new MeasurementException("Both measurements do not have the same dimenion powers!");        
      }
    }
  }
  
  
  public double getValue(Measurement multipleOf) {
    checkCompatibility(multipleOf);
    
    // Having checked everything, get the value
    double scaledValue = this.systemValue / multipleOf.getSystemValue();
    
    return scaledValue;
  }

  /**
   * The tolerance used when inspecting the {@link org.xspace.mx.Unit#getPower() power} of a {@link org.xspace.mx.Unit}.
   */  
  protected static double TOL = 1e-6; // Tolerance
  // protected double TOL = 1e-3; // Tolerance
  
  /**
   * A description of the measurement
   *
   * @return A string describing the Measurement
   */
  public String toString() {
    StringBuffer sb = new StringBuffer();
    sb.append(this.systemValue);
    sb.append(" ");
    boolean bAtLeastOneDim = false;
    for (int i = 0; i < dims.length; i++) {
      if (Math.abs((dims[i].getPower() - 0)) > TOL) {// BUG 2003-08-28 Use Math.abs() to account for only negative exponents
        bAtLeastOneDim = true;
        break; // Stop checking and proceed
      }  
    }
    
    if (bAtLeastOneDim) {
      sb.append("[ ");
      for (int i = 0; i < dims.length; i++) {
        double pow = dims[i].getPower();
        if (Math.abs(pow - 0) > TOL) {
          sb.append(dims[i].getType().toString());
          sb.append("^");
          sb.append(pow);
          sb.append(" ");
        }
      }
      sb.append("]");
    }
    
    return sb.toString();
  }  
  
  
  /**
   * This private method is called even by subclasses when they are being deserialized
   */
  private void readObject(java.io.ObjectInputStream ois) throws java.io.IOException, ClassNotFoundException {    
    ois.defaultReadObject();

    int iVersion = ois.readInt();
    switch (iVersion) {
      case 1:
        systemValue = ois.readDouble(); // System Value
        dims = (Unit[]) ois.readObject(); // Array of dimensions        
        break;
        
      default:
        // Unexpected
        // Do nothing special        
        break;
    }
    
  }
  
  private void writeObject(java.io.ObjectOutputStream oos) throws java.io.IOException {
    oos.defaultWriteObject();

    final int SERIALIZATION_VERSION = 1;
    oos.writeInt(SERIALIZATION_VERSION); // Version Number
    oos.writeDouble(systemValue); // System Value
    oos.writeObject(dims); // Array of dimensions
  }
  
}
