/*
 * Copyright (C) 2002-2006 Michel Santos (michel@xspace.org)
 * Licensed under the Academic Free License version 3.0
 * Please see the LICENSE.txt for more information.
 *
 * Created April 30, 2003, 9:50 PM
 * Updated December 1, 2003 21:24
 * - Units are now expressed as being "standard" types of measurements
 *   such as length, mass, time, etc. whenever possible
 *
 * Updated April 23, 2004
 * - Added utility functions to check for compatible units
 * - Added utility functions to cast objects as Length units, Time units, etc.
 */

package org.xspace.mx.physics.immutable;

import org.xspace.mx.Measurement;

/**
 * Helpler class to provide easy access to <i>standard</i> measurements.
 * <i>These measurements are immutable.</i>
 *
 * @author Michel Santos
 * @since 0.5
 */
public class Units {
  
  /** Creates a new instance of Units */
  private Units() {
  }

  // Angular constants
  public static final Angle rad = new Radian();
  public static final Angle deg = new Degree();
  public static final Angle rev = new Revolution();
  
  
  // Length constants
  public static final Length m = new Meter();
  public static final Length cm = new Centimeter();
  public static final Length km = new Kilometer();
  public static final Length AU = new AstronomicalUnit();
  public static final Length ft = new Foot();
  public static final Length in = new Inch();
  public static final Length mi = new Mile();
  
  // Time constants
  public static final Time s = new Second();
  public static final Time min = new Minute();  
  public static final Time hr = new Hour();
  public static final Time day = new Day();
  
  // Mass constants
  public static final Mass kg = new Kilogram();
  
  // Force constants
  public static final Measurement N = new Newton();
  public static final Measurement lbf = new PoundForce();

  
  
  public static boolean isLengthUnit(org.xspace.mx.physics.immutable.BaseUnit mx) {
    return mx.isSameMeasurementType(Units.m);
  }  
  public static boolean isTimeUnit(org.xspace.mx.physics.immutable.BaseUnit mx) {
    return mx.isSameMeasurementType(Units.s);
  }
  public static boolean isForceUnit(org.xspace.mx.physics.immutable.BaseUnit mx) {
    return mx.isSameMeasurementType(Units.N);
  }  
  private static final Measurement MX_METERS_PER_SECOND = Units.m.div(Units.s);
  public static boolean isSpeedUnit(org.xspace.mx.physics.immutable.BaseUnit mx) {
    return mx.isSameMeasurementType(MX_METERS_PER_SECOND);
  }
  
  
  public static boolean isLengthUnit(org.xspace.mx.Measurement mx) {
    return org.xspace.mx.physics.SI.isSameMeasurementType(mx, Units.m);
  }
  public static boolean isTimeUnit(org.xspace.mx.Measurement mx) {
    return org.xspace.mx.physics.SI.isSameMeasurementType(mx, Units.s);
  }
  public static boolean isForceUnit(org.xspace.mx.Measurement mx) {
    return org.xspace.mx.physics.SI.isSameMeasurementType(mx, Units.N);
  }
  public static boolean isSpeedUnit(org.xspace.mx.Measurement mx) {
    return org.xspace.mx.physics.SI.isSameMeasurementType(mx, MX_METERS_PER_SECOND);
  }
  

  
  public static org.xspace.mx.physics.immutable.Length asLengthUnit(org.xspace.mx.Measurement mx) {
    if (mx instanceof org.xspace.mx.physics.immutable.Length) {
      return (org.xspace.mx.physics.immutable.Length)mx;
    } else {
      return new Meter(mx);
    }
  }
  
  public static org.xspace.mx.physics.immutable.Time asTimeUnit(org.xspace.mx.Measurement mx) {
    if (mx instanceof org.xspace.mx.physics.immutable.Time) {
      return (org.xspace.mx.physics.immutable.Time)mx;
    } else {
      return new Second(mx);
    }
  }  
  
  public static org.xspace.mx.physics.immutable.Force asForceUnit(org.xspace.mx.Measurement mx) {
    if (mx instanceof org.xspace.mx.physics.immutable.Force) {
      return (org.xspace.mx.physics.immutable.Force)mx;
    } else {
      return new Newton(mx);
    }
  }  
    
}